import matplotlib
import os
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np


def calc_mean_and_std(arr):
    return np.mean(arr), np.std(arr)


####


# Configure
result_file = "comparison_result.dat"
result_file = "comparison_result_astro1.dat"
name_testcase = "Astro1"
img_dir = "scaling_plots"

# Readout file
results = []
with open(result_file, "r") as f:
    for line in f:
        res = list(eval(line.strip()))
        if len(res) == 6:
            res.append(res[-2] / res[-1])
        results.append(res)

# make dataframe
headers = [
    "cores",
    "total_systems",
    "total_time_sequentially",
    "total_time_multiprocessing",
    "ratio",
]

df = pd.DataFrame(results)
df.columns = headers

# Select unique amounts
unique_amt_cores = df["cores"].unique()
unique_amt_systems = df["total_systems"].unique()

# Create dictionary with calculated means and stdevs etc
calculated_results = []
for i in unique_amt_cores:
    for j in unique_amt_systems:
        (
            total_time_sequential_list,
            total_time_multiprocessing_list,
            total_ratio_list,
        ) = ([], [], [])

        for res in results:
            if (res[0] == i) & (res[1] == j):
                total_time_sequential_list.append(res[2])
                total_time_multiprocessing_list.append(res[3])
                total_ratio_list.append(res[4])

        if (
            (total_time_sequential_list)
            and (total_time_multiprocessing_list)
            and (total_ratio_list)
        ):
            # calculate stuff
            mean_time_sequential, std_sequential = calc_mean_and_std(
                np.array(total_time_sequential_list)
            )
            mean_time_multiprocessing, std_multiprocessing = calc_mean_and_std(
                np.array(total_time_multiprocessing_list)
            )
            mean_ratio, std_ratio = calc_mean_and_std(np.array(total_ratio_list))

            # make dict
            res_dict = {
                "cores": i,
                "systems": j,
                "mean_time_sequential": mean_time_sequential,
                "mean_time_multiprocessing": mean_time_multiprocessing,
                "mean_ratio": mean_ratio,
                "std_sequential": std_sequential,
                "std_multiprocessing": std_multiprocessing,
                "std_ratio": std_ratio,
                "total_runs": len(total_time_sequential_list),
            }

            calculated_results.append(res_dict)

#############################################################################################################################
# Plot speed up  and efficiency.
x_position_shift = 0
y_position_shift = -0.05
max_speedup = 0
# https://stackoverflow.com/questions/46323530/matplotlib-plot-two-x-axes-one-linear-and-one-with-logarithmic-ticks
fig, ax1 = plt.subplots()
ax2 = ax1.twinx()
for amt_systems in unique_amt_systems:

    cores = []
    speedup = []
    std = []
    efficiency = []

    for el in calculated_results:
        if el["systems"] == amt_systems:

            cores.append(el["cores"] + x_position_shift)
            speedup.append(el["mean_ratio"])
            std.append(el["std_ratio"])
            efficiency.append(el["mean_ratio"] / el["cores"])

            if el["mean_ratio"] > max_speedup:
                max_speedup = el["mean_ratio"]

            # add number of runs its based on
            ax1.text(
                el["cores"] + x_position_shift + 0.01,
                el["mean_ratio"] + y_position_shift,
                el["total_runs"],
            )

    ax1.errorbar(
        cores,
        speedup,
        std,
        linestyle="None",
        marker="^",
        label="Speed up & efficiency of {} systems".format(amt_systems),
    )
    ax2.plot(cores, efficiency, alpha=0.5)
    x_position_shift += 0.1

ax1.set_title(
    "Speed up ratio vs amount of cores for different amounts of systems on {}".format(
        name_testcase
    )
)
ax1.set_xlabel("Amount of cores used")
ax1.set_ylabel("Speed up ratio (time_linear/time_parallel)")

ax1.set_xlim(0, max(unique_amt_cores) + 4)
ax1.set_ylim(0, max_speedup + 2)
ax2.set_ylim(0, 1)

ax1.grid()
ax1.legend(loc=4)
fig.savefig(os.path.join(img_dir, "speedup_scaling_{}.{}".format(name_testcase, "png")))
fig.savefig(os.path.join(img_dir, "speedup_scaling_{}.{}".format(name_testcase, "pdf")))
fig.savefig(os.path.join(img_dir, "speedup_scaling_{}.{}".format(name_testcase, "eps")))
plt.show()

#####
# Plot run time
# fig = plt.figure()
# ax = fig.add_subplot(111)

# color_options = ['blue', 'green', 'red', 'black', 'yellow', 'magenta']

# x_position_shift = 0
# y_position_shift = -0.05

# colors = [color_options[i] for i in range(len(unique_amt_systems))]
# for i, amt_systems in enumerate(unique_amt_systems):

#     cores = []

#     mean_times_linear = []
#     mean_times_multiprocessing = []
#     std_linear = []
#     std_mp = []

#     for el in calculated_results:
#         if el['systems']==amt_systems:
#             cores.append(el['cores'] + x_position_shift)

#             mean_times_linear.append(el['mean_time_sequential'])
#             std_linear.append(el['std_sequential'])

#             mean_times_multiprocessing.append(el['mean_time_multiprocessing'])
#             std_mp.append(el['std_multiprocessing'])

#             # # add number of runs its based on
#             # plt.text(el['cores'] + x_position_shift+0.01, el['mean_ratio']+y_position_shift, el['total_runs'])

#     ax.errorbar(cores, mean_times_linear, std_linear, color=colors[i], linewidth=1, marker='P', label='{} systems linear run'.format(amt_systems))
#     ax.errorbar(cores, mean_times_multiprocessing, std_multiprocessing, linewidth=1, color=colors[i],  marker='d', label='{} systems MP run'.format(amt_systems))
#     x_position_shift += 0.1

# ax.set_title("Total time for sequential and MP for different amounts of systems on {}".format(name_testcase))
# ax.set_xlabel("Amount of cores used")
# ax.set_ylabel("Total time taken (s)")

# ax.set_xlim(0, max(unique_amt_cores) + 20)
# ax.set_yscale('log')
# ax.grid()
# ax.legend()

# fig.savefig(os.path.join(img_dir, 'total_time_scaling_{}.{}'.format(name_testcase, 'png')))
# fig.savefig(os.path.join(img_dir, 'total_time_scaling_{}.{}'.format(name_testcase, 'pdf')))
# fig.savefig(os.path.join(img_dir, 'total_time_scaling_{}.{}'.format(name_testcase, 'eps')))

# plt.show()
